require 'spec_helper'
require 'bunny'

describe Banter::Publisher do
  let(:routing_key) { "test/logger" }
  let(:publisher) { Banter::Publisher.new }
  let(:context) { {unique_id: "1234", orig_ip_address: "127.0.0.1"}}
  let(:contents) { Message::new.serialize(context, routing_key, {"a"=>"b"})}
  let(:channel_mocker) {
    topic = double("::Bunny::Exchange", :publish=> true )
    if publish_fails
      topic.stub(:on_return).and_yield({routing_key: 'a.b.c'}, {some: 'property'}, contents).and_return(false)
    else
      topic.stub(:on_return).and_return true
    end
    double("::Bunny::Channel", :topic=> topic)
  }
  let(:publish_fails){ false }
  let(:session_mock) { Bunny::Session.new }

  before do
    session_mock
    publisher
    allow(Banter::RabbitLogger).to receive(:log_publish)
    allow(::Bunny::Session).to receive(:new).and_return(session_mock)
    allow(Banter::RabbitLogger).to receive(:log)
  end

  describe "#delay_messages" do

    context "no duplicates", :fixit=>true do
      let(:result) {
        publisher.delay_messages { operations } }

      let(:operations) {
        publisher.publish(context, "first", "one")
        publisher.publish(context, "second", "two")
        }

      it "should attempt to publish twice" do
        result
        expect(Banter::RabbitLogger).to have_received(:log).with(Logger::INFO, /Publishing all messages from delayed publish/)
        expect(Banter::RabbitLogger).to have_received(:log).with(Logger::INFO, /to delayed publish with/).exactly(2).times
      end

    end

    context "Publish fails", publish_fails: true do
      let(:publish_fails){ true }
      let(:result) { publisher.start }

      specify {expect { result }.not_to raise_error }
    end

    context "duplicate entries" do
      let(:result) { publisher.delay_messages { operations } }

      let(:operations) {
        publisher.publish(context, "first", "one")
        publisher.publish(context, "second", "two")
        publisher.publish(context, "second", "three")
        }

      before do
        allow(publisher).to receive(:execute_publish).and_call_original
        result
      end

      it "should attempt to publish three times" do
        expect(publisher).to have_received(:execute_publish).exactly(3).times
      end
    end

    describe "adds to the queue when delaying messages" do
      let(:result) { publisher.delay_messages { operations } }
      let(:operations) {
        publisher.publish(context, "first", "one")
        publisher.publish(context, "second", "two")
      }

      before do
        allow(publisher).to receive(:add_message).and_call_original
        allow(publisher).to receive(:execute_publish).and_call_original
        result
      end

      it "should have added to the message list twice that will be published last" do
        expect(publisher).to have_received(:add_message).exactly(2).times
      end

      it "should have added to the message list twice that will be published last" do
        expect(publisher).to have_received(:execute_publish).exactly(2).times
      end

    end

  end

  describe "#teardown" do
    context "should not fail" do
      before do
        allow(Banter::RabbitLogger).to receive(:log).with(Logger::WARN, /RabbitMQ teardown failed/)
      end

      it do
        expect{publisher.send(:teardown)}.not_to raise_error
      end

      it do
        publisher.send(:teardown)
        expect(Banter::RabbitLogger).to have_received(:log).exactly(1).times
      end
    end

  end

end
