require 'spec_helper'

class TestSubscriber < Banter::Subscriber
  subscribe_to "some.test.key"
end

describe Banter::Server::ClientQueueListener do
  describe "#message_received" do

    subject { -> { queue_listener.message_received({}, { routing_key: routing_key }, { context: context, payload: payload }) } }
    let(:subscriber_config) { Hashie::Mash.new(
      topic: "banter",
      exchange: "banter",
      dead_letter_exchange: "banter-dlx",
      routing_key: routing_key,
      queue_name: "test_queue",
      ttl: 100,
      durable: true,
      pool_size: 100
    )}

    let(:queue_listener) { Banter::Server::ClientQueueListener.new(Klass) }
    let(:routing_key) { 'some.test.key' }
    let(:context) { { app: 'test' } }
    let(:payload) { { hello: 'moto' } }
    let(:context_before) { }
    let(:validators) { [] }
    let(:banter_worker) { FactoryGirl.create(:banter_worker, process_name: 'test') }
    let!(:application_name) { "testapplication" }
    let(:subscriber_options) {{}}

    before do
      Banter::Configuration.application_name = application_name
      context_before
      Object.const_set :Klass, Class.new(Banter::Subscriber)
      validators.each do |validator|
        Klass.validates_payload_with validator
      end
      Klass.subscribe_to "test.foo", subscriber_options

      klass_instance

      allow(Klass).to receive(:new).and_return(klass_instance)
    end

    after do
      Object.send :remove_const, :Klass
    end

    def safe_call
      subject.call
    rescue=>e
      @raised_error = e
      nil
    end

    context "Normal execution" do
      let!(:klass_instance) {
        instance = Klass.new({}, {}, {}, {})
        allow(instance).to receive(:perform!)
        instance
      }

      before do
        safe_call unless respond_to? :no_call
      end

      context "test exception" do
        let(:no_call) { true }
        it { should_not raise_exception }
      end

      context "call" do
        it { expect(klass_instance).to have_received(:perform!).with(payload) }
      end

    end

    context "Exception during execution" do
      let!(:klass_instance) {
        instance = Klass.new({}, {}, {}, {})
        allow(instance).to receive(:perform!) {
          raise StandardError.new("test exception")
        }
        instance
      }
      let(:prehook) {}

      before do
        prehook
        allow(Banter::RabbitLogger).to receive(:log).and_call_original
        safe_call
      end

      context "without a notifier" do
        let(:prehook) {
          allow(STDOUT).to receive(:puts).and_call_original
        }
        it { expect(@raised_error).not_to be_nil }
      end

      context "with a notifier" do
        let(:prehook) {
          Banter::Configuration.notifier = TestNotifier
          allow(TestNotifier).to receive(:notify).and_call_original
        }

        it { expect(@raised_error).not_to be_nil }
        it { expect(TestNotifier).to have_received(:notify) }
        it { expect(TestNotifier.instance.error.to_s).to match(/test exception/) }
        it { expect(TestNotifier.instance.hash.keys).to match_array([:delivery_info, :envelope, :properties]) }
      end
    end

    context "Payload error" do
      let!(:klass_instance) {
        instance = Klass.new({}, {}, {},{})
        allow(instance).to receive(:perform!).and_raise(Banter::PayloadValidationError.new("validation failed"))
        instance
      }
      let(:result) { subject.call }
      let(:prehook) {
        Banter::Configuration.notifier = TestNotifier
        allow(TestNotifier).to receive(:notify).and_call_original
      }


      before do
        prehook
        safe_call
      end

      let!(:validators) { [lambda { |payload| false }] }
      it { should raise_exception }
      it { expect(TestNotifier).to have_received(:notify) }
      it { expect(TestNotifier.instance.error.to_s).to match(/validation failed/) }
      it { expect(TestNotifier.instance.hash.keys).to match_array([:delivery_info, :envelope, :properties]) }
    end

    context "reprocessing message" do
      subject { -> { queue_listener.message_received({}, { routing_key: routing_key }, Hashie::Mash.new({ context: context, payload: payload, dead_letter: { retries: 1, id: 553} }) ) } }
      let(:subscriber_options) {{ dead_letter_queue_name: "deadletter" }}
      let(:test_method) { raise StandardError.new("test exception") }
      let!(:klass_instance) {
        instance = Klass.new({}, {}, {}, {})
        allow(instance).to receive(:perform!) {
          test_method
        }
        instance
      }

      before do
        klass_instance
        allow(Banter::Publisher.instance).to receive(:execute_publish)
        safe_call
      end

      context "succeed in reprocess" do
        let(:test_method) { {} }

        it "should have called the execute_publish method of the queue publisher" do
          # Marked the message with a completed flag and send to a queue for dead letter.  also, the exchange is correct being empty.
          hash_data = { context: {app:"test"}, dead_letter: { completed: 1,  id: 553, retries: 1 }, payload: { hello: "moto"} }
          val = Hashie::Mash.new(hash_data)
          expect(Banter::Publisher.instance).to have_received(:execute_publish).with("deadletter", val, true )
        end

      end

      context "failed to reprocess" do
        it "should not have called the execute_publish method of the queue publisher" do
          expect(Banter::Publisher.instance).not_to have_received(:execute_publish)
        end

      end

    end
  end
end