require 'spec_helper'


describe Banter::Server::RabbitMQSubscriber do
  let(:routing_key) { "fake.route"}
  let(:subscriber_config) { Hashie::Mash.new(
    topic: "banter",
    exchange: "banter",
    dead_letter_exchange: "banter-dlx",
    routing_key: routing_key,
    queue_name: "test_queue",
    ttl: 100,
    durable: true,
    pool_size: 100
  )}
  let(:subscriber){ Banter::Server::RabbitMQSubscriber.new( subscriber_config ) }
  let(:delivery_info) { Hashie::Mash.new({ :delivery_tag=>1, :routing_key=> routing_key}) }
  let(:properties) { {} }
  let(:contents) { File.read( File.join(File.dirname(__FILE__), "../../fixtures/utf_message.json")) }


  def do_nothing(delivery, props, content)
  end

  def raise_method(delivery, props, content)
    raise StandardError.new("Bad Subscriber")
  end

  describe "#process_message" do
    let(:result) {
      subscriber.process_message(delivery_info, properties, contents)
    }

    before do
      subscriber.instance_variable_set(:@channel, double("channel", :ack=>true, :reject=>true))
      allow(subscriber.channel).to receive(:ack)
      allow(subscriber.channel).to receive(:reject)
    end


    context "subscriber has an error" do
      before do
        Banter::Configuration.notifier = TestNotifier
        subscriber.instance_variable_set(:@callback_block, method(:raise_method) )
      end


      it "should not raise any errors" do
        expect{ result }.not_to raise_error
      end

      it "should have rejected the message" do
        result
        expect(subscriber.channel).to have_received(:reject).with(1, false)
      end

      it "should have generated a notification" do
        result
        expect(TestNotifier.instance.error.to_s).to match(/Bad Subscriber/)
      end
    end

    context "subscriber works correctly" do
      before do
        subscriber.instance_variable_set(:@callback_block, method(:do_nothing) )
        allow(TestNotifier).to receive(:notify).and_call_original
      end

      it "should not raise any errors" do
        expect{ result }.not_to raise_error
      end

      it "should have acknowledged the message" do
        result
        expect(subscriber.channel).to have_received(:ack).with(1)
      end

      it "should not have trigggered a notification" do
        result
        expect(TestNotifier).not_to have_received(:notify)
      end
    end

    context "logger has problems" do

      before do
        Banter::Configuration.notifier = TestNotifier
        subscriber.instance_variable_set(:@callback_block, method(:do_nothing) )
        allow(Banter::RabbitLogger).to receive(:log_receive).and_raise("Cannot log")
        allow(Banter::RabbitLogger).to receive(:log_complete)
        allow(TestNotifier).to receive(:notify).and_call_original
      end

      it "should not raise any errors" do
        expect{ result }.not_to raise_error
      end

      it "should have rejected the message" do
        result
        expect(subscriber.channel).to have_received(:reject).with(1, false)
      end

      it "should not have logged a complete message" do
        result
        expect(Banter::RabbitLogger).not_to have_received(:log_complete)
      end


      it "should have generated a notification" do
        result
        expect(TestNotifier).to have_received(:notify).exactly(1).times
      end


    end

  end

  describe "#notify_error" do
    let(:error) { StandardError.new("Error Message")}
    let(:consumer) { Object.new}
    let(:result) { subscriber.notify_error(error, consumer)}

    before do
      Banter::Configuration.notifier = TestNotifier
      allow(TestNotifier).to receive(:notify).and_call_original
    end

    it "should not raise any errors" do
      expect{result}.not_to raise_error
    end

    it "should inform of the problem" do
      result
      expect(TestNotifier).to have_received(:notify).exactly(1).times
    end

  end

  describe "#initialize" do
    context "when pool size is missing" do
      let(:subscriber_config) { Hashie::Mash.new(
        topic: "banter",
        exchange: "banter",
        dead_letter_exchange: "banter-dlx",
        routing_key: routing_key,
        queue_name: "test_queue",
        ttl: 100,
        durable: true
      )}

      it "should not call basic_qos" do
        expect(subscriber.pool_size).to eq(0)
      end

    end

  end

end
